@@ -61,6 +61,8 @@ from traitlets import (
     HasDescriptors,
     CUnicode,
 )
+from traitlets.utils import cast_unicode
+
 
 def change_dict(*ordered_values):
     change_names = ('name', 'old', 'new', 'owner', 'type')
@@ -1462,16 +1464,20 @@ class UnicodeTrait(HasTraits):
 
     value = Unicode('unicode')
 
+
 class TestUnicode(TraitTestBase):
 
     obj = UnicodeTrait()
 
     _default_value = 'unicode'
     _good_values   = ['10', '-10', '10L', '-10L', '10.1',
-                      '-10.1', '', 'string', "€"]
+                      '-10.1', '', 'string', "€", b"bytestring"]
     _bad_values    = [10, -10, 10.1, -10.1, 1j,
                       [10], ['ten'], {'ten': 10},(10,), None]
 
+    def coerce(self, v):
+        return cast_unicode(v)
+
 
 class ObjectNameTrait(HasTraits):
     value = ObjectName("abc")
@@ -1495,7 +1501,7 @@ class TestDottedObjectName(TraitTestBase):
     _default_value = "a.b"
     _good_values = ["A", "y.t", "y765.__repr__", "os.path.join"]
     _bad_values = [1, "abc.€", "_.@", ".", ".abc", "abc.", ".abc.", None]
-    
+
     _good_values.append("t.þ")
 
 
@@ -2652,7 +2658,7 @@ def _from_string_test(traittype, s, expected):
     if isinstance(traittype, TraitType):
         trait = traittype
     else:
-        trait = traittype()
+        trait = traittype(allow_none=True)
     if isinstance(s, list):
         cast = trait.from_string_list
     else:
@@ -2667,107 +2673,149 @@ def _from_string_test(traittype, s, expected):
 
 
 @pytest.mark.parametrize(
-    "s, expected", [("xyz", "xyz"), ("1", "1"), ('"xx"', "xx"), ("'abc'", "abc"),]
+    "s, expected",
+    [("xyz", "xyz"), ("1", "1"), ('"xx"', "xx"), ("'abc'", "abc"), ("None", None)],
 )
 def test_unicode_from_string(s, expected):
     _from_string_test(Unicode, s, expected)
 
 
 @pytest.mark.parametrize(
-    "s, expected", [("xyz", "xyz"), ("1", "1"), ('"xx"', "xx"), ("'abc'", "abc"),]
+    "s, expected",
+    [("xyz", "xyz"), ("1", "1"), ('"xx"', "xx"), ("'abc'", "abc"), ("None", None)],
 )
 def test_cunicode_from_string(s, expected):
     _from_string_test(CUnicode, s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('x', ValueError),
-    ('1', 1),
-    ('123', 123),
-    ('2.0', ValueError),
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [("xyz", b"xyz"), ("1", b"1"), ('b"xx"', b"xx"), ("b'abc'", b"abc"), ("None", None)],
+)
+def test_bytes_from_string(s, expected):
+    _from_string_test(Bytes, s, expected)
+
+
+@pytest.mark.parametrize(
+    "s, expected",
+    [("xyz", b"xyz"), ("1", b"1"), ('b"xx"', b"xx"), ("b'abc'", b"abc"), ("None", None)],
+)
+def test_cbytes_from_string(s, expected):
+    _from_string_test(CBytes, s, expected)
+
+
+@pytest.mark.parametrize(
+    "s, expected",
+    [("x", ValueError), ("1", 1), ("123", 123), ("2.0", ValueError), ("None", None)],
+)
 def test_int_from_string(s, expected):
     _from_string_test(Integer, s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('x', ValueError),
-    ('1', 1.0),
-    ('123.5', 123.5),
-    ('2.5', 2.5),
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [("x", ValueError), ("1", 1.0), ("123.5", 123.5), ("2.5", 2.5), ("None", None)],
+)
 def test_float_from_string(s, expected):
     _from_string_test(Float, s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('x', ValueError),
-    ('1', 1.0),
-    ('123.5', 123.5),
-    ('2.5', 2.5),
-    ('1+2j', 1+2j),
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [
+        ("x", ValueError),
+        ("1", 1.0),
+        ("123.5", 123.5),
+        ("2.5", 2.5),
+        ("1+2j", 1 + 2j),
+        ("None", None),
+    ],
+)
 def test_complex_from_string(s, expected):
     _from_string_test(Complex, s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('true', True),
-    ('TRUE', True),
-    ('1', True),
-    ('0', False),
-    ('False', False),
-    ('false', False),
-    ('1.0', ValueError),
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [
+        ("true", True),
+        ("TRUE", True),
+        ("1", True),
+        ("0", False),
+        ("False", False),
+        ("false", False),
+        ("1.0", ValueError),
+        ("None", None),
+    ],
+)
 def test_bool_from_string(s, expected):
     _from_string_test(Bool, s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('{}', {}),
-    ('1', TraitError),
-    ('{1: 2}', {1: 2}),
-    ('{"key": "value"}', {"key": "value"}),
-    ('x', TraitError),
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [
+        ("{}", {}),
+        ("1", TraitError),
+        ("{1: 2}", {1: 2}),
+        ('{"key": "value"}', {"key": "value"}),
+        ("x", TraitError),
+        ("None", None),
+    ],
+)
 def test_dict_from_string(s, expected):
     _from_string_test(Dict, s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('[]', []),
-    ('[1, 2, "x"]', [1, 2, 'x']),
-    (["1", "x"], ["1", "x"])
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [
+        ("[]", []),
+        ('[1, 2, "x"]', [1, 2, "x"]),
+        (["1", "x"], ["1", "x"]),
+        (["None"], None),
+    ],
+)
 def test_list_from_string(s, expected):
     _from_string_test(List, s, expected)
 
 
-@pytest.mark.parametrize('s, expected, value_trait', [
-    (["1", "2", "3"], [1, 2, 3], Integer()),
-    (["x"], ValueError, Integer()),
-    (["1", "x"], ["1", "x"], Unicode())
-])
-def test_list_from_string(s, expected, value_trait):
+@pytest.mark.parametrize(
+    "s, expected, value_trait",
+    [
+        (["1", "2", "3"], [1, 2, 3], Integer()),
+        (["x"], ValueError, Integer()),
+        (["1", "x"], ["1", "x"], Unicode()),
+        (["None"], [None], Unicode(allow_none=True)),
+    ],
+)
+def test_list_items_from_string(s, expected, value_trait):
     _from_string_test(List(value_trait), s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('x', 'x'),
-    ('mod.submod', 'mod.submod'),
-    ('not an identifier', TraitError),
-    ('1', '1'),
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [
+        ("x", "x"),
+        ("mod.submod", "mod.submod"),
+        ("not an identifier", TraitError),
+        ("1", "1"),
+        ("None", None),
+    ],
+)
 def test_object_from_string(s, expected):
     _from_string_test(DottedObjectName, s, expected)
 
 
-@pytest.mark.parametrize('s, expected', [
-    ('127.0.0.1:8000', ('127.0.0.1', 8000)),
-    ('host.tld:80', ('host.tld', 80)),
-    ('host:notaport', ValueError),
-    ('127.0.0.1', ValueError),
-])
+@pytest.mark.parametrize(
+    "s, expected",
+    [
+        ("127.0.0.1:8000", ("127.0.0.1", 8000)),
+        ("host.tld:80", ("host.tld", 80)),
+        ("host:notaport", ValueError),
+        ("127.0.0.1", ValueError),
+        ("None", None),
+    ],
+)
 def test_tcp_from_string(s, expected):
     _from_string_test(TCPAddress, s, expected)
